/*
    Copyright (C) 1995-2000 AROS - The Amiga Research OS
    $Id: break.c,v 1.4 2008/05/05 22:42:42 piru Exp $
 
    Desc: Break - send a signal to a process.
    Lang: english
*/

/*****************************************************************************
 
    NAME
	Break
 
    FORMAT
	Break <process> [ALL|C|D|E|F]
 
    SYNOPSIS
	PROCESS/N,PORT,ALL/S,C/S,D/S,E/S,F/S
 
    LOCATION
	Workbench:c
 
    FUNCTION
	BREAK sends a signal to another process to get its attention.
	The argument |PROCESS| specifies the number of the process that
	you wish to send the signal to. You can find out process numbers
	with the STATUS command.
	You can also specify a public port name and send signal's to the
	port's task.
 
	You can send a combination of the flags CTRL-C, CTRL-D, CTRL-E
	and CTRL-F. By default, only the CTRL-C flag is sent.
 
	The effect of using the BREAK command is the same as selecting
	the console window of a process and pressing the relevant key
	combination.
 
	The normal meaning of the keys is:
	    CTRL-C	-	Halt a process
	    CTRL-D	-	Halt a shell script
	    CTRL-E	-	Close a process' window
	    CTRL-F	-	Make active the process' window
 
	Not all programs respond to these signals, however most should
	respond to CTRL-C.
 
    EXAMPLE
 
	1.SYS:> BREAK 1
 
	    Send the CTRL-C signal to the process numbered 1.
 
	1.SYS:> BREAK 4 E
 
	    Send the CTRL-E signal to the process numbered 4.

	1.SYS:> BREAK port="FUBAR" C
 
	    Send the CTRL-C signal to the process used in the public msgport
	    "FUBAR".
 
    SEE ALSO
 
	STATUS
 
**************************************************************************/

#include <exec/types.h>
#include <exec/memory.h>
#include <exec/tasks.h>
#include <exec/execbase.h>
#include <dos/dos.h>
#include <dos/dosextens.h>
#include <dos/exall.h>
#include <dos/rdargs.h>
#include <proto/exec.h>
#include <proto/dos.h>

#define ARG_TEMPLATE	"PROCESS/N,PORT,ALL/S,C/S,D/S,E/S,F/S"
#define ARG_PROCESS	0
#define ARG_PORT	1
#define ARG_ALL		2
#define ARG_C		3
#define ARG_D		4
#define ARG_E		5
#define ARG_F		6
#define TOTAL_ARGS	7

#include "break_version.h"

static const char version[] = VERSTAG;

static const char exthelp[] =
    "Break : Set the attention flags of a DOS task\n"
    "\tPROCESS/N  Process to set flags for\n"
    "\tPORT       Portname for the Process to set flags for\n"
    "\tALL/S      Set ALL attention flags\n"
    "\tC/S        Set the CTRL-C flag\n"
    "\tD/S        Set the CTRL-D flag\n"
    "\tE/S        Set the CTRL-E flag\n"
    "\tF/S        Set the CTRL-F flag\n";

int main(void)
{
    struct ExecBase *SysBase=*((struct ExecBase**) 4);
    struct DosLibrary	*DOSBase;
    struct Process *pr;
    struct RDArgs *rd, *rda = NULL;
    IPTR args[TOTAL_ARGS] = { 0, 0, FALSE, FALSE, FALSE, FALSE, FALSE };
    BPTR errStream;
    ULONG mask = 0;
    int error = 0;

    if ((DOSBase = (struct DosLibrary *)OpenLibrary("dos.library", 37)))
    {
        errStream = Output();

        #if 0
        pr = (struct Process *)FindTask(NULL);
        if (pr->pr_CES != NULL)
            errStream = pr->pr_CES;
        #endif
        pr = NULL;

        if((rda = AllocDosObject(DOS_RDARGS, NULL)))
        {
            rda->RDA_ExtHelp = (STRPTR)exthelp;

            if((rd = ReadArgs(ARG_TEMPLATE, (LONG *)args, rda)))
            {
                Forbid();

                if (args[ARG_PROCESS] && *(ULONG *) args[ARG_PROCESS] != 0)
                {
                     pr = FindCliProc(*(ULONG*) args[ARG_PROCESS]);
                     if (!pr)
                     {
                         if ((SysBase->LibNode.lib_Version == 50 && SysBase->LibNode.lib_Revision >= 45) ||
                              SysBase->LibNode.lib_Version > 50)
                         {
                              pr = (struct Process *) FindTaskByPID(*(ULONG*) args[ARG_PROCESS]);
                         }
                     }
                }
                else if (args[ARG_PORT])
                {
                    struct MsgPort *MyPort;
                    /* NOTE: We're within Forbid... */
                    if ((MyPort = (struct MsgPort*) FindPort((char*) args[ARG_PORT])))
                    {
                        pr =(struct Process*) MyPort->mp_SigTask;
                    }
                }

                if( pr != NULL )
                {
                    /* Figure out the mask of flags to send. */
                    if( args[ARG_ALL] )
                        mask = SIGBREAKF_CTRL_C | SIGBREAKF_CTRL_D
                               | SIGBREAKF_CTRL_E | SIGBREAKF_CTRL_F;
                    else
                        mask =  (args[ARG_C] ? SIGBREAKF_CTRL_C : 0)
                                |	(args[ARG_D] ? SIGBREAKF_CTRL_D : 0)
                                |	(args[ARG_E] ? SIGBREAKF_CTRL_E : 0)
                                |	(args[ARG_F] ? SIGBREAKF_CTRL_F : 0);

                    if (mask == 0)
                        mask = SIGBREAKF_CTRL_C;

                    Signal((struct Task *)pr, mask);

                    Permit();
                }
                else
                {
                    Permit();

                    /* There is no relevant error code, OBJECT_NOT_FOUND
                        is a filesystem error, so we can't use that... */

                    if (args[ARG_PROCESS])
                    {
                        VFPrintf(errStream, "Break: Process %ld does not exist.\n", (APTR) args[ARG_PROCESS]);
                    }
                    else if (args[ARG_PORT])
                    {
                        FPrintf(errStream, "Break: Port \"%s\" does not exist.\n", (LONG) args[ARG_PORT]);
                    }
                    else
                    {
                        FPuts(errStream, "Break: Either PROCESS or PORT is required.\n");
                    }
                    error = -1;
                }

                FreeArgs(rd);
            } /* ReadArgs() ok */
            else
                error = IoErr();

            FreeDosObject(DOS_RDARGS, rda);
        } /* Got rda */
        else
            error = IoErr();

        if( (error != 0) && (error != -1) )
        {
        #if 0
            UBYTE tmp[200];
            Fault(error, "Break", tmp, sizeof(tmp));
            FPuts(errStream, tmp); FPuts(errStream, "\n");
        #else
            PrintFault(error, "Break");
        #endif

            error = RETURN_FAIL;
        }
        else
        {
            error = (error) ? RETURN_FAIL : RETURN_OK;  /* "-1" indicates failing too */
            SetIoErr(0);
        }
        CloseLibrary((struct Library *)DOSBase);
    }
    else
	error = RETURN_FAIL;

    return(error);
}
